from notebook.services.contents.filemanager import FileContentsManager

import s3fs

from traitlets import (
    HasTraits,
    Any,
    Bool,
    Dict,
    Instance,
    List,
    TraitError,
    Type,
    Unicode,
    validate,
    default,
    Instance
)

import json

from .bookstore_config import BookstoreSettings

class BookstoreContentsArchiver(FileContentsManager):
    """
    Archives notebooks to S3 on save
    """

    def __init__(self, *args, **kwargs):
        super(FileContentsManager, self).__init__(*args, **kwargs)
        # opt ourselves into being part of the Jupyter App that should have Bookstore Settings applied
        self.settings = BookstoreSettings(parent=self)

        self.log.info("Archiving notebooks to {}".format(self.full_prefix))

        self.fs = s3fs.S3FileSystem(key=self.settings.s3_access_key_id,
                                    secret=self.settings.s3_secret_access_key,
                                    client_kwargs={
                                        "endpoint_url": self.settings.s3_endpoint_url,
                                        "region_name": self.settings.s3_region_name
                                    },
                                    config_kwargs={},
                                    s3_additional_kwargs={})

    @property
    def delimiter(self):
        """It's a slash! Normally this could be configurable. This leaves room for that later,
        keeping it centralized for now"""
        return "/"

    @property
    def full_prefix(self):
        """Full prefix: bucket + workspace prefix"""
        return self.delimiter.join([self.settings.s3_bucket, self.settings.workspace_prefix])

    def s3_path(self, path):
        """compute the s3 path based on the bucket, prefix, and the path to the notebook"""
        return self.delimiter.join([self.full_prefix, path])

    def run_pre_save_hook(self, model, path, **kwargs):
        """Store notebook to S3 when saves happen
        """
        if model['type'] != 'notebook':
            return

        # TODO: store the hash of the notebook to not write on every save
        notebook_contents = json.dumps(model['content'])

        full_path = self.s3_path(path)

        # write to S3
        # TODO: Write to S3 asynchronously to not block other server operations
        with self.fs.open(full_path, mode='wb') as f:
            f.write(notebook_contents.encode('utf-8'))



from traitlets import (
    Any,
    Bool,
    Dict,
    Instance,
    List,
    TraitError,
    Type,
    Unicode,
    validate,
    default,
)

from traitlets.config import LoggingConfigurable

class BookstoreSettings(LoggingConfigurable):
    """The same settings to be shared across archival, publishing, and scheduling
    """

    workspace_prefix = Unicode("workspace", help="Prefix for the live workspace notebooks").tag(config=True)
    published_prefix = Unicode("published", help="Prefix for published notebooks").tag(config=True)

    ## S3 Settings for the S3 backed storage (other implementations can add on below)
    # Allowed to not set these as we can pick up IAM roles instead
    s3_access_key_id = Unicode(
        help="S3/AWS access key ID", allow_none=True, default_value=None).tag(
            config=True, env="JPYNB_S3_ACCESS_KEY_ID")
    s3_secret_access_key = Unicode(
        help="S3/AWS secret access key", allow_none=True, default_value=None).tag(
            config=True, env="JPYNB_S3_SECRET_ACCESS_KEY")

    s3_endpoint_url = Unicode(
        "https://s3.amazonaws.com", help="S3 endpoint URL").tag(
            config=True, env="JPYNB_S3_ENDPOINT_URL")
    s3_region_name = Unicode(
        "us-east-1", help="Region name").tag(
            config=True, env="JPYNB_S3_REGION_NAME")
    s3_bucket = Unicode(
        "bookstore", help="Bucket name to store notebooks").tag(
            config=True, env="JPYNB_S3_BUCKET")



import json
from notebook.utils import url_path_join
from notebook.base.handlers import APIHandler, \
    AuthenticatedFileHandler
from tornado import web

from ._version import get_versions
version = get_versions()['version']

class BookstoreVersionHandler(APIHandler):
    """Returns the version of bookstore currently running. Used mostly to lay foundations
    for this package though frontends can use this endpoint for feature detection.
    """
    @web.authenticated
    def get(self):
        self.finish(json.dumps({"bookstore": True, "version": version}))

def load_jupyter_server_extension(nb_app):
    web_app = nb_app.web_app
    host_pattern = '.*$'
    base_bookstore_pattern = url_path_join(web_app.settings['base_url'],
                    '/api/bookstore')

    web_app.add_handlers(host_pattern, [
        (base_bookstore_pattern, BookstoreVersionHandler),
    ])




from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

from .handlers import load_jupyter_server_extension

def _jupyter_server_extension_paths():
    return [dict(module="bookstore")]

from .archive import BookstoreContentsArchiver

from .bookstore_config import BookstoreSettings



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""""
setup.py

Note: Do a version check for IPython.
    IPython v6+ no longer supports Python 2.
    If Python 2, install ipython 5.x.

See:
https://packaging.python.org/tutorials/packaging-projects/
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject

"""
from __future__ import print_function
import os
import sys
from os import path
from setuptools import setup

# io.open is needed for projects that support Python 2.7
# It ensures open() defaults to text mode with universal newlines,
# and accepts an argument to specify the text encoding
# Python 3 only projects can skip this import
from io import open

import versioneer

python_2 = sys.version_info[0] == 2


def read(fname):
    with open(fname, 'rU' if python_2 else 'r') as fhandle:
        return fhandle.read()


req_path = os.path.join(os.path.dirname('__file__'), 'requirements.txt')
required = [req.strip() for req in read(req_path).splitlines() if req.strip()]

test_req_path = os.path.join(os.path.dirname('__file__'), 'requirements-dev.txt')
test_required = [req.strip() for req in read(test_req_path).splitlines() if req.strip()]
extras_require = {"test": test_required, "dev": test_required}

pip_too_old = False
pip_message = ''

try:
    import pip

    pip_version = tuple([int(x) for x in pip.__version__.split('.')[:3]])
    pip_too_old = pip_version < (9, 0, 1)
    if pip_too_old:
        # pip is too old to handle IPython deps gracefully
        pip_message = (
            'Your pip version is out of date. bookstore requires pip >= 9.0.1. \n'
            'pip {} detected. Please install pip >= 9.0.1.'.format(pip.__version__)
        )
except ImportError:
    pip_message = (
        'No pip detected; we were unable to import pip. \n'
        'To use bookstore, please install pip >= 9.0.1.'
    )
except Exception:
    pass

if pip_message:
    print(pip_message, file=sys.stderr)
    sys.exit(1)


# Get the long description from the README file
here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='bookstore',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description='Storage Workflows for Notebooks',
    author='nteract contributors',
    author_email='nteract@googlegroups.com',
    license='BSD',
    # Note that this is a string of words separated by whitespace, not a list.
    keywords='jupyter storage nteract notebook',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/nteract/bookstore',
    packages=['bookstore'],
    install_requires=required,
    extras_require=extras_require,
    entry_points={},
    project_urls={
        'Documentation': 'https://github.com/nteract/bookstore/#todo',
        'Funding': 'https://nteract.io',
        'Source': 'https://github.com/nteract/bookstore/',
        'Tracker': 'https://github.com/nteract/bookstore/issues',
    },
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2.7',
    ],
)
